from __future__ import division, print_function

import time
import numpy as np
from sklearn.linear_model import Ridge


def strf_correlation(strf1, strf2, max_delay=10):
    """
        Computes the dot product between two STRFs across in a way that's invariant to frequency differences and time lags up to max_delay.
    """

    if strf1.shape != strf2.shape:
        raise ValueError('STRF shapes do not match: %s != %s' % (str(strf1.shape), str(strf2.shape)))
    nf = strf1.shape[0]

    time_shifts = np.arange(-max_delay, max_delay+1, 1, dtype='int')
    freq_shifts = np.arange(nf, dtype='int')
    all_ccs = np.zeros([len(time_shifts), len(freq_shifts)])
    s1 = strf1 / np.abs(strf1).max()
    s2 = strf2 / np.abs(strf2).max()
    ns1 = s1 - s1.mean()
    for i,d in enumerate(time_shifts):
        for j,f in enumerate(freq_shifts):
            #roll along frequency axis
            rs2 = np.roll(s2, f, axis=0)
            #shift and zero along time axis
            rs2 = np.roll(rs2, d, axis=1)
            if d > 0:
                rs2[:, :d] = 0.0
            elif d < 0:
                rs2[:, d:] = 0.0
            ns2 = rs2 - rs2.mean()
            cc = (ns1 * ns2).mean() / ( s1.std() * s2.std() )
            all_ccs[i, j] = cc

    all_ccs_abs = np.abs(all_ccs)
    mi = np.unravel_index(all_ccs_abs.argmax(), all_ccs.shape)
    argmax_delay,argmax_freq = time_shifts[mi[0]], freq_shifts[mi[1]]
    return argmax_delay, argmax_freq, all_ccs[mi[0], mi[1]]


def strf_mps(strf, fstep, sample_rate, half=False):

    nchannels,strflen = strf.shape
    fstrf = np.fliplr(strf)
    mps = np.fft.fftshift(np.fft.fft2(fstrf))
    amps = np.real(mps * np.conj(mps))

    #Obtain labels for frequency axis
    dwf = np.zeros([nchannels])
    fcircle = 1.0 / fstep
    for i in range(nchannels):
        dwf[i] = (i/float(nchannels))*fcircle
        if dwf[i] > fcircle/2.0:
            dwf[i] -= fcircle

    dwf = np.fft.fftshift(dwf)
    if dwf[0] > 0.0:
        dwf[0] = -dwf[0]

    #Obtain labels for time axis
    fcircle = sample_rate
    dwt = np.zeros([strflen])
    for i in range(strflen):
        dwt[i] = (i/float(strflen))*fcircle
        if dwt[i] > fcircle/2.0:
            dwt[i] -= fcircle

    dwt = np.fft.fftshift(dwt)
    if dwt[0] > 0.0:
        dwt[0] = -dwt[0]

    if half:
        halfi = np.where(dwf == 0.0)[0][0]
        amps = amps[halfi:, :]
        dwf = dwf[halfi:]

    return dwf,dwt,amps


def strf_conv(X, strf, lags, bias):
    """ Performs a convolution between the multivariate time series "input" with the spatio-temporal receptive
        field ("strf").

        :param X: The input time series, of shape (num_time_points,num_features)
        :param strf: The receptive field, of shape (num_features, num_lags)
        :param lags: The integer-valued lag for each column of strf
        :param bias: The scalar offset

        :return: A time series y of shape (num_time_points)
    """

    input_T = np.matrix(X)
    nsamps = input_T.shape[0]
    #print 'input_T.shape=',input_T.shape
    #print 'time_lags.shape=',time_lags.shape
    #print 'filter.shape=',filter.shape

    a = np.zeros( [nsamps, 1] )
    for k,ti in enumerate(lags):
        #print '\tti=%d' % ti
        #print '\tk=%d, filter[:, k].shape=' % k,filter[:, k].shape
        at = input_T * strf[:, k].reshape(strf.shape[0], 1)
        if ti >= 0:
            if ti > 0:
                at = at[:-ti]
            #print '\tat.shape=',at.shape
            a[ti:] += at
        else:
            offset = ti % nsamps
            a[:offset] += at[-ti:]

    return a.squeeze() + bias


def make_toeplitz(input, lags, include_bias=True, fortran_style=False):
    """
        Assumes input is of dimensionality nt x nf, where nt is the number of time points and
        nf is the number of features.

        lags is an array of integers, representing the time lag from zero. a negative time lag points to the future,
        positive to the past.
    """

    nt = input.shape[0]
    nf = input.shape[1]
    d = len(lags)

    if fortran_style:
        A = np.zeros([nt, nf*d+include_bias], order='F')
    else:
        A = np.zeros([nt, nf*d+include_bias])
    if include_bias:
        A[:, -1] = 1.0 # the bias term

    all_indices = np.arange(d*nf)

    #compute the channel corresponding to each parameter in the reshaped (flattened) filter
    channel_indices = np.floor(all_indices / float(d)).astype('int')

    #compute the lag index corresponding to each parameter in the reshaped (flattened) filter
    lag_indices = all_indices % d
    #print 'lag_indices=',lag_indices

    for k,i in enumerate(all_indices):
        #get lag and channel corresponding to this index
        lag_index = lag_indices[i]
        #print 'k=%d, i=%d, lag_index=%d' % (k, i, lag_index)
        lag = lags[lag_index]
        channel_to_get = channel_indices[i]

        if lag == 0:
            A[:, k] = input[:, channel_to_get]
        else:
            #shift time series for this channel up or down depending on lag
            if lag > 0:
                A[lag:, k] = input[:-lag, channel_to_get]
            else:
                A[:lag, k] = input[-lag:, channel_to_get] #note that lag is negative
    return A


def fit_strf_lasso(input, output, lags, lambda1=1.0, lambda2=1.0, num_threads=-1):
    try:
        import spams
    except ImportError:
        print('Cannot import spams! No lasso for you...')
        return

    #convert the input into a toeplitz-like matrix
    stime = time.time()
    A = make_toeplitz(input, lags, include_bias=True, fortran_style=True)
    etime = time.time() - stime
    print('[fit_strf_lasso] Time to make Toeplitz matrix: %d seconds' % etime)

    fy = np.asfortranarray(output.reshape(len(output), 1))
    #print 'fy.shape=',fy.shape
    #print 'fA.shape=',fA.shape

    #fit the STRF
    stime = time.time()
    fit_params = spams.lasso(fy, A, mode=2, lambda1=lambda1, lambda2=lambda2, numThreads=num_threads)
    etime = time.time() - stime
    print('[fit_strf_lasso] Time to fit STRF: %d seconds' % etime)

    #reshape the STRF so that it makes sense
    nt = input.shape[0]
    nf = input.shape[1]
    d = len(lags)
    strf = np.array(fit_params[:-1].todense()).reshape([nf, d])
    bias = fit_params[-1].todense()[0, 0]

    return strf,bias


def fit_strf_ridge(input, output, lags, alpha=1.0, verbose=False):
    """ Fit a spatio-temporal receptive field (STRF). A STRF maps input stimuli
        into a scalar output variable. A simple ridge regression algorithm is used
        to fit the STRF.

    :param input: A numpy array of shape (num_time_points, num_spatial_channels).
    :param output: A numpy array of shape (num_time_points).
    :param lags: An array of integers that specify which time points should be included. For example,
                 to fit a STRF that uses only the stimulus information at time t to predict the output
                 at time t, specify lags=[0]. To fit a STRF that uses the stimulus information at
                 time t and the previous 10 time points, specify lags=range(0, 11).
    :param alpha: The regularization parameter for ridge regression. Try a bunch!
    :param verbose:
    :return: strf,bias: strf is a numpy array of shape (num_spatial_channels,len(lags)), which is the
            receptive field. bias is a scalar.
    """

    #convert the input into a toeplitz-like matrix
    if verbose:
        nt,nf = input.shape
        nelems = nt*nf*len(lags)
        mem = (nelems*8.) / 1024.**2
        print('[fit_strf_ridge] estimated size of toeplitz matrix: %d MB' % mem)
    stime = time.time()
    A = make_toeplitz(input, lags, include_bias=False)
    etime = time.time() - stime
    if verbose:
        print('[fit_strf_ridge] Time to make Toeplitz matrix: %d seconds' % etime)

    #fit the STRF
    stime = time.time()

    #rr = Ridge(alpha=alpha, copy_X=False, fit_intercept=True)
    rr = Ridge(alpha=alpha, fit_intercept=True)
    rr.fit(A, output)
    etime = time.time() - stime
    if verbose:
        print('[fit_strf_ridge] Time to fit STRF: %d seconds' % etime)

    #reshape the STRF so that it makes sense
    nt = input.shape[0]
    nf = input.shape[1]
    d = len(lags)
    strf = np.array(rr.coef_).reshape([nf, d])
    bias = rr.intercept_

    return strf,bias



#!/bin/env python
# Written by Greg Ver Steeg
# See readme.pdf for documentation
# Or go to http://www.isi.edu/~gregv/npeet.html

from __future__ import division, print_function

import scipy.spatial as ss
from scipy.special import digamma
from math import log
import numpy.random as nr
import numpy as np
import random


# CONTINUOUS ESTIMATORS

def entropy(x, k=3, base=2):
    """ The classic K-L k-nearest neighbor continuous entropy estimator
        x should be a list of vectors, e.g. x = [[1.3], [3.7], [5.1], [2.4]]
        if x is a one-dimensional scalar and we have four samples
    """
    assert k <= len(x) - 1, "Set k smaller than num. samples - 1"
    d = len(x[0])
    N = len(x)
    intens = 1e-10  # small noise to break degeneracy, see doc.
    x = [list(p + intens * nr.rand(len(x[0]))) for p in x]
    tree = ss.cKDTree(x)
    nn = [tree.query(point, k + 1, p=float('inf'))[0][k] for point in x]
    const = digamma(N) - digamma(k) + d * log(2)
    return (const + d * np.mean(map(log, nn))) / log(base)


def mi(x, y, k=3, base=2):
    """ Mutual information of x and y
        x, y should be a list of vectors, e.g. x = [[1.3], [3.7], [5.1], [2.4]]
        if x is a one-dimensional scalar and we have four samples
    """
    assert len(x) == len(y), "Lists should have same length"
    assert k <= len(x) - 1, "Set k smaller than num. samples - 1"
    intens = 1e-10  # small noise to break degeneracy, see doc.
    x = [list(p + intens * nr.rand(len(x[0]))) for p in x]
    y = [list(p + intens * nr.rand(len(y[0]))) for p in y]
    points = zip2(x, y)
    # Find nearest neighbors in joint space, p=inf means max-norm
    tree = ss.cKDTree(points)
    dvec = [tree.query(point, k + 1, p=float('inf'))[0][k] for point in points]
    a, b, c, d = avgdigamma(x, dvec), avgdigamma(y, dvec), digamma(k), digamma(len(x))
    return (-a - b + c + d) / log(base)


def cmi(x, y, z, k=3, base=2):
    """ Mutual information of x and y, conditioned on z
        x, y, z should be a list of vectors, e.g. x = [[1.3], [3.7], [5.1], [2.4]]
        if x is a one-dimensional scalar and we have four samples
    """
    assert len(x) == len(y), "Lists should have same length"
    assert k <= len(x) - 1, "Set k smaller than num. samples - 1"
    intens = 1e-10  # small noise to break degeneracy, see doc.
    x = [list(p + intens * nr.rand(len(x[0]))) for p in x]
    y = [list(p + intens * nr.rand(len(y[0]))) for p in y]
    z = [list(p + intens * nr.rand(len(z[0]))) for p in z]
    points = zip2(x, y, z)
    # Find nearest neighbors in joint space, p=inf means max-norm
    tree = ss.cKDTree(points)
    dvec = [tree.query(point, k + 1, p=float('inf'))[0][k] for point in points]
    a, b, c, d = avgdigamma(zip2(x, z), dvec), avgdigamma(zip2(y, z), dvec), avgdigamma(z, dvec), digamma(k)
    return (-a - b + c + d) / log(base)


def kldiv(x, xp, k=3, base=2):
    """ KL Divergence between p and q for x~p(x), xp~q(x)
        x, xp should be a list of vectors, e.g. x = [[1.3], [3.7], [5.1], [2.4]]
        if x is a one-dimensional scalar and we have four samples
    """
    assert k <= len(x) - 1, "Set k smaller than num. samples - 1"
    assert k <= len(xp) - 1, "Set k smaller than num. samples - 1"
    assert len(x[0]) == len(xp[0]), "Two distributions must have same dim."
    d = len(x[0])
    n = len(x)
    m = len(xp)
    const = log(m) - log(n - 1)
    tree = ss.cKDTree(x)
    treep = ss.cKDTree(xp)
    nn = [tree.query(point, k + 1, p=float('inf'))[0][k] for point in x]
    nnp = [treep.query(point, k, p=float('inf'))[0][k - 1] for point in x]
    return (const + d * np.mean(map(log, nnp)) - d * np.mean(map(log, nn))) / log(base)


# DISCRETE ESTIMATORS
def entropyd(sx, base=2):
    """ Discrete entropy estimator
        Given a list of samples which can be any hashable object
    """
    return entropyfromprobs(hist(sx), base=base)


def midd(x, y):
    """ Discrete mutual information estimator
        Given a list of samples which can be any hashable object
    """
    return -entropyd(zip(x, y)) + entropyd(x) + entropyd(y)


def cmidd(x, y, z):
    """ Discrete mutual information estimator
        Given a list of samples which can be any hashable object
    """
    return entropyd(zip(y, z)) + entropyd(zip(x, z)) - entropyd(zip(x, y, z)) - entropyd(z)


def hist(sx):
    # Histogram from list of samples
    d = dict()
    for s in sx:
        d[s] = d.get(s, 0) + 1
    return map(lambda z: float(z) / len(sx), d.values())


def entropyfromprobs(probs, base=2):
    # Turn a normalized list of probabilities of discrete outcomes into entropy (base 2)
    return -sum(map(elog, probs)) / log(base)


def elog(x):
    # for entropy, 0 log 0 = 0. but we get an error for putting log 0
    if x <= 0. or x >= 1.:
        return 0
    else:
        return x * log(x)


# MIXED ESTIMATORS
def micd(x, y, k=3, base=2, warning=True):
    """ If x is continuous and y is discrete, compute mutual information
    """
    overallentropy = entropy(x, k, base)

    n = len(y)
    word_dict = dict()
    for sample in y:
        word_dict[sample] = word_dict.get(sample, 0) + 1. / n
    yvals = list(set(word_dict.keys()))

    mi = overallentropy
    for yval in yvals:
        xgiveny = [x[i] for i in range(n) if y[i] == yval]
        if k <= len(xgiveny) - 1:
            mi -= word_dict[yval] * entropy(xgiveny, k, base)
        else:
            if warning:
                print("Warning, after conditioning, on y=", yval, " insufficient data. Assuming maximal entropy in this case.")
            mi -= word_dict[yval] * overallentropy
    return mi  # units already applied


# UTILITY FUNCTIONS
def vectorize(scalarlist):
    """ Turn a list of scalars into a list of one-d vectors
    """
    return [(x, ) for x in scalarlist]


def shuffle_test(measure, x, y, z=False, ns=200, ci=0.95, **kwargs):
    """ Shuffle test
        Repeatedly shuffle the x-values and then estimate measure(x, y, [z]).
        Returns the mean and conf. interval ('ci=0.95' default) over 'ns' runs.
        'measure' could me mi, cmi, e.g. Keyword arguments can be passed.
        Mutual information and CMI should have a mean near zero.
    """
    xp = x[:]  # A copy that we can shuffle
    outputs = []
    for i in range(ns):
        random.shuffle(xp)
        if z:
            outputs.append(measure(xp, y, z, **kwargs))
        else:
            outputs.append(measure(xp, y, **kwargs))
    outputs.sort()
    return np.mean(outputs), (outputs[int((1. - ci) / 2 * ns)], outputs[int((1. + ci) / 2 * ns)])


# INTERNAL FUNCTIONS

def avgdigamma(points, dvec):
    # This part finds number of neighbors in some radius in the marginal space
    # returns expectation value of <psi(nx)>
    N = len(points)
    tree = ss.cKDTree(points)
    avg = 0.
    for i in range(N):
        dist = dvec[i]
        # subtlety, we don't include the boundary point,
        # but we are implicitly adding 1 to kraskov def bc center point is included
        num_points = len(tree.query_ball_point(points[i], dist - 1e-15, p=float('inf')))
        avg += digamma(num_points) / N
    return avg


def zip2(*args):
    # zip2(x, y) takes the lists of vectors and makes it a list of vectors in a joint space
    # E.g. zip2([[1], [2], [3]], [[4], [5], [6]]) = [[1, 4], [2, 5], [3, 6]]
    return [sum(sublist, []) for sublist in zip(*args)]

if __name__ == "__main__":
    print("NPEET: Non-parametric entropy estimation toolbox. See readme.pdf for details on usage.")



"""
Generate quasi-random numbers using Halton, Hammerseley, and Sobol sequences.
"""
from __future__ import division, print_function

import numpy as np

from soundsig.thirdparty.sobol_lib import i4_sobol_generate


def quasirand(M, N, type='sobol', spherical=False):
    """Generate N quasi-random M dimensional vectors.

        Args:
            M (int): the dimension of each random vector
            N (int): the number of random vectors to generate
            type (str): the type of sequence to generate ['sobol']
            spherical (bool): whether or not to generate points on a sphere (False)

        Returns:
            R (np.ndarray): an MxN matrix of quasi-random numbers.

    """

    Mused = M - int(spherical)

    R = None
    if type == 'sobol':
        R = i4_sobol_generate(Mused, N, 1)

    if spherical:
        # project M-1 dimensional random numbers onto an M dimensional sphere
        # taken from equation 3.2 of N. Rehman and D. P. Mandic "Multivariate empirical mode decomposition" (2014)

        #rescale the random numbers so they lie in (-pi, pi]
        pi_eps = 1e-6
        R *= 2*np.pi - pi_eps
        R -= np.pi - pi_eps

        # add extra dimension to end
        R = np.vstack([R, np.ones([N])])

        #update each dimension
        for k in reversed(range(M)):

            #construct the new row, which will be a product of sines and one cosine
            new_row = np.ones([N])
            for j in range(k):
                new_row *= np.sin(R[j, :])

            #multiply the last term, a cosine for all but the very last row
            if k < M-1:
                new_row *= np.cos(R[k, :])

            #assign the product of sines and cosines to the current row, which is not used in updating the rows
            #that come before it
            R[k, :] = new_row

    return R




from setuptools import setup

setup(
    name = 'soundsig',
    version = '1.5',
    packages = ['soundsig'],
    description = 'Sound and Signal Analysis Tools for Bioacousticians and Auditory Neurophysiologists',
    author = 'Frederic Theunissen',
    author_email = 'theunissen@berkeley.edu',
    url = 'https://github.com/theunissenlab/soundsig', 
    keywords = 'bioacoustics biosound vocalization auditory',
    classifiers = ['Development Status :: 4 - Beta',
                   'Programming Language :: Python :: 3.0'],
    install_requires = ['numpy',
                      'scipy',
                      'matplotlib',
                      'tables',
                      'h5py',
                      'mne',
                      'nitime',
                      'pandas',
                      'scikit-learn']
)



# -*- coding: utf-8 -*-
"""
Created on Wed Sep 14 22:16:23 2016

downloaded from http://scipy-cookbook.readthedocs.io/items/Matplotlib_HintonDiagrams.html?highlight=hinton%20diagrams
"""
from __future__ import division, print_function

import numpy as N
import pylab as P

def _blob(x,y,area,colour):
    """
    Draws a square-shaped blob with the given area (< 1) at
    the given coordinates.
    """
    hs = N.sqrt(area) / 2
    xcorners = N.array([x - hs, x + hs, x + hs, x - hs])
    ycorners = N.array([y - hs, y - hs, y + hs, y + hs])
    P.fill(xcorners, ycorners, colour, edgecolor=colour)

def hinton(W, maxWeight=None):
    """
    Draws a Hinton diagram for visualizing a weight matrix. 
    Temporarily disables matplotlib interactive mode if it is on, 
    otherwise this takes forever.
    """
    reenable = False
    if P.isinteractive():
        P.ioff()
    P.clf()
    height, width = W.shape
    if not maxWeight:
        maxWeight = 2**N.ceil(N.log(N.max(N.abs(W)))/N.log(2))
        
    P.fill(N.array([0,width,width,0]),N.array([0,0,height,height]),'gray')
    P.axis('off')
    P.axis('equal')
    for x in range(width):
        for y in range(height):
            _x = x+1
            _y = y+1
            w = W[y,x]
            if w > 0:
                _blob(_x - 0.5, height - _y + 0.5, min(1,w/maxWeight),'white')
            elif w < 0:
                _blob(_x - 0.5, height - _y + 0.5, min(1,-w/maxWeight),'black')
    if reenable:
        P.ion()
    # P.show()
    return P.gcf()



from __future__ import division, print_function

import copy
import operator
from matplotlib import patches
from matplotlib.colors import LinearSegmentedColormap

import numpy as np

import matplotlib.pyplot as plt
from scipy.interpolate import interp1d

from .stats import compute_R2


def multi_plot(the_data_list, plot_func, title=None, nrows=4, ncols=5, figsize=None, output_pattern=None,
               transpose=False, facecolor='gray', hspace=0.20, wspace=0.20, bottom=0.02, top=0.95, right=0.97, left=0.03):

    nsp = 0
    fig = None
    fig_num = 0
    plots_per_page = nrows*ncols

    data_list = the_data_list
    overflow_index = 0
    if transpose:
        data_list = [None]*len(data_list)
        for k in range(len(the_data_list)):
            page_offset = int(float(k) / plots_per_page)*plots_per_page
            if len(the_data_list) - page_offset < plots_per_page:
                new_index = page_offset + overflow_index
                overflow_index += 1
            else:
                sp = k % plots_per_page
                row = sp % nrows
                col = int(float(sp) / nrows)
                new_index = page_offset + row*ncols + col
            print('nsp=%d, k=%d, sp=%d, page_offset=%d, row=%d, col=%d, new_index=%d' %
                  (len(the_data_list), k, sp, page_offset, row, col, new_index))
            data_list[new_index] = the_data_list[k]

    for pdata in data_list:
        if nsp % plots_per_page == 0:
            if output_pattern is not None and fig is not None:
                #save the current figure
                ofile = output_pattern % fig_num
                plt.savefig(ofile, facecolor=fig.get_facecolor(), edgecolor='none')
                plt.close('all')
            fig = plt.figure(figsize=figsize, facecolor=facecolor)
            fig_num += 1
            fig.subplots_adjust(top=top, bottom=bottom, right=right, left=left, hspace=hspace, wspace=wspace)
            if title is not None:
                plt.suptitle(title + (" (%d)" % fig_num))

        nsp += 1
        sp = nsp % plots_per_page
        ax = fig.add_subplot(nrows, ncols, sp)
        plot_func(pdata, ax)

    #save last figure
    if fig is not None and output_pattern is not None:
        ofile = output_pattern % fig_num
        plt.savefig(ofile, facecolor=fig.get_facecolor(), edgecolor='none')
        plt.close('all')


def plot_pairwise_analysis(data_mat, feature_columns, dependent_column, column_names):
    """
        Does a basic pairwise correlation analysis between features and a dependent variable,
        meaning it plots a scatter plot with a linear curve fit through it, with the R^2.
        Then it plots a correlation matrix for all features and the dependent variable.

        data_mat: an NxM matrix, where there are N samples, M-1 features, and 1 dependent variable.

        feature_columns: the column indices of the features in data_mat that are being examined

        dependent_column: the column index of the dependent variable in data_mat

        column_names: a list of len(feature_columns)+1 feature/variable names. The last element is
                      the name of the dependent variable.
    """

    plot_data = list()
    for k,fname in enumerate(column_names[:-1]):
        fi = feature_columns[k]

        pdata = dict()
        pdata['x'] = data_mat[:, fi]
        pdata['y'] = data_mat[:, dependent_column]
        pdata['xlabel'] = column_names[fi]
        pdata['ylabel'] = column_names[-1]
        pdata['R2'] = compute_R2(pdata['x'], pdata['y'])
        plot_data.append(pdata)

    #sort by R^2
    plot_data.sort(key=operator.itemgetter('R2'), reverse=True)
    multi_plot(plot_data, plot_pairwise_scatter, title=None, nrows=3, ncols=3)

    all_columns = copy.copy(feature_columns)
    all_columns.append(dependent_column)

    C = np.corrcoef(data_mat[:, all_columns].transpose())

    Cy = C[:, -1]
    corr_list = [(column_names[k], np.abs(Cy[k]), Cy[k]) for k in range(len(column_names)-1)]
    corr_list.sort(key=operator.itemgetter(1), reverse=True)

    print('Correlations  with %s' % column_names[-1])
    for cname,abscorr,corr in corr_list:
        print('\t%s: %0.6f' % (cname, corr))

    fig = plt.figure()
    plt.subplots_adjust(top=0.99, bottom=0.15, left=0.15)
    ax = fig.add_subplot(1, 1, 1)
    fig.autofmt_xdate(rotation=45)
    im = ax.imshow(C, interpolation='nearest', aspect='auto', vmin=-1.0, vmax=1.0, origin='lower')
    plt.colorbar(im)
    ax.set_yticks(range(len(column_names)))
    ax.set_yticklabels(column_names)
    ax.set_xticks(range(len(column_names)))
    ax.set_xticklabels(column_names)


def plot_pairwise_scatter(plot_data, ax):

    x = plot_data['x']
    y = plot_data['y']
    if 'R2' not in plot_data:
        R2 = compute_R2(x, y)
    else:
        R2 = plot_data['R2']
    slope,bias = np.polyfit(x, y, 1)
    sp = (x.max() - x.min()) / 25.0
    xrng = np.arange(x.min(), x.max(), sp)

    clr = '#aaaaaa'
    if 'color' in plot_data:
        clr = plot_data['color']
    ax.plot(x, y, 'o', mfc=clr)
    ax.plot(xrng, slope*xrng + bias, 'k-')
    ax.set_title('%s: R2=%0.2f' % (plot_data['xlabel'], R2))
    if 'ylabel' in plot_data:
        ax.set_ylabel(plot_data['ylabel'])
    ax.set_ylim(y.min(), y.max())


def plot_histogram_categorical(x, xname='x', y=None, yname='y', color='g'):
    """
        Makes a histogram of the variable x, which is an array of categorical variables in their native representation
        (string or intger) . If a dependent continuous variable y is specified, it will make another plot which
        is a bar graph showing the mean and standard deviation of the continuous variable y.
    """

    ux = np.unique(x)
    xfracs = np.array([(x == xval).sum() for xval in ux]) / float(len(x))

    nsp = 1 + (y is not None)
    ind = range(len(ux))

    fig = plt.figure()
    ax = fig.add_subplot(nsp, 1, 1)
    ax.bar(ind, xfracs, facecolor=color, align='center', ecolor='black')
    ax.set_xticks(ind)
    ax.set_xticklabels(ux)
    ax.set_xlabel(xname)
    ax.set_ylabel('Fraction of Samples')

    if y is not None:
        y_per_x = dict()
        for xval in ux:
            indx = x == xval
            y_per_x[xval] = y[indx]

        ystats = [ (xval, y_per_x[xval].mean(), y_per_x[xval].std()) for xval in ux]
        ystats.sort(key=operator.itemgetter(0), reverse=True)

        xvals = [x[0] for x in ystats]
        ymeans = np.array([x[1] for x in ystats])
        ystds = np.array([x[2] for x in ystats])

        ax = fig.add_subplot(nsp, 1, 2)
        ax.bar(ind, ymeans, yerr=ystds, facecolor=color, align='center', ecolor='black')
        ax.set_xticks(ind)
        ax.set_xticklabels(xvals)
        #fig.autofmt_xdate()
        ax.set_ylabel('Mean %s' % yname)
        ax.set_xlabel(xname)
        ax.set_ylim(0, (ymeans+ystds).max())


def whist(x, **kwds):
    return plt.hist(x, weights=np.ones([len(x)]) / float(len(x)), **kwds)


def plot_confusion_matrix_single(pdata, ax):
    plt.imshow(pdata['cmat'], interpolation='nearest', aspect='auto', origin='upper', vmin=0, vmax=1)
    plt.title('p=%0.3f' % pdata['p'])


def make_phase_image(amp, phase, normalize=True, saturate=True, threshold=True):
    """
        Turns a phase matrix into an image to be plotted with imshow.
    """

    import husl

    nelectrodes,d = amp.shape
    alpha = copy.deepcopy(amp)
    if normalize:
        max_amp = np.percentile(amp, 98)
        alpha = alpha / max_amp

    img = np.zeros([nelectrodes, d, 4], dtype='float32')

    #set the alpha and color for the bins
    if saturate:
        alpha[alpha > 1.0] = 1.0 #saturate
    if threshold:
        alpha[alpha < 0.05] = 0.0 #nonlinear threshold

    cnorm = ((180.0 / np.pi) * phase).astype('int')
    for j in range(nelectrodes):
        for ti in range(d):
            #img[j, ti, :3] = husl.husl_to_rgb(cnorm[j, ti], 99.0, 50.0) #use HUSL color space: https://github.com/boronine/pyhusl/tree/v2.1.0
            img[j, ti, :3] = husl.husl_to_rgb(cnorm[j, ti], 99.0, 61.0) #use HUSL color space: https://github.com/boronine/pyhusl/tree/v2.1.0

    img[:, :, 3] = alpha

    return img


def draw_husl_circle():
    """ Draw an awesome circle whose angle is colored using the HUSL color space. The HUSL color space is circular, so
        it's useful for plotting phase. This figure could serve as a "color circle" as opposed to a color bar.
    """

    import husl

    #generate a bunch of points on the circle
    theta = np.arange(0.0, 2*np.pi, 1e-3)

    plt.figure()

    radii = np.arange(0.75, 1.0, 1e-2)
    for t in theta:
        x = radii*np.cos(t)
        y = radii*np.sin(t)

        a = (180.0/np.pi)*t
        c = husl.husl_to_rgb(a, 99.0, 50.0)
        plt.plot(x, y, c=c)
    plt.xlim(-1.25, 1.25)
    plt.ylim(-1.25, 1.25)

    plt.show()


def register_husl_colormap(nsegs=90):

    import husl

    cdict = {'red':list(), 'green':list(), 'blue':list()}

    r0,g0,b0 = husl.husl_to_rgb(0, 99.0, 50)
    cdict['red'].append((0., r0, r0))
    cdict['blue'].append((0., b0, b0))
    cdict['green'].append((0., g0, g0))

    inc = 360. / nsegs
    for k in range(nsegs):
        d = (k+1)*inc
        x = (k+1) / float(nsegs)
        r,g,b = husl.husl_to_rgb(d, 99.0, 50.0)
        cdict['red'].append((x, r, r))
        cdict['green'].append((x, g, g))
        cdict['blue'].append((x, b, b))

    cm = LinearSegmentedColormap('HUSL', cdict)
    plt.register_cmap(cmap=cm)


def custom_legend(colors, labels, linestyles=None):
    """ Creates a list of matplotlib Patch objects that can be passed to the legend(...) function to create a custom
        legend.

    :param colors: A list of colors, one for each entry in the legend. You can also include a linestyle, for example: 'k--'
    :param labels:  A list of labels, one for each entry in the legend.
    """

    if linestyles is not None:
        assert len(linestyles) == len(colors), "Length of linestyles must match length of colors."

    h = list()
    for k,(c,l) in enumerate(zip(colors, labels)):
        clr = c
        ls = 'solid'
        if linestyles is not None:
            ls = linestyles[k]
        patch = patches.Patch(color=clr, label=l, linestyle=ls)
        h.append(patch)
    return h


def grouped_boxplot(data, group_names=None, subgroup_names=None, ax=None, subgroup_colors=None,
                    box_width=0.6, box_spacing=1.0, legend_loc='upper right'):
    """ Draws a grouped boxplot. The data should be organized in a hierarchy, where there are multiple
        subgroups for each main group.

    :param data: A dictionary of length equal to the number of the groups. The key should be the
                group name, the value should be a list of arrays. The length of the list should be
                equal to the number of subgroups.
    :param group_names: (Optional) The group names, should be the same as data.keys(), but can be ordered.
    :param subgroup_names: (Optional) Names of the subgroups.
    :param subgroup_colors: A list specifying the plot color for each subgroup.
    :param ax: (Optional) The axis to plot on.
    """

    if group_names is None:
        group_names = data.keys()

    if ax is None:
        ax = plt.gca()
    plt.sca(ax)

    nsubgroups = np.array([len(v) for v in data.values()])
    assert len(np.unique(nsubgroups)) == 1, "Number of subgroups for each property differ!"
    nsubgroups = nsubgroups[0]

    if subgroup_colors is None:
        subgroup_colors = list()
        for k in range(nsubgroups):
            subgroup_colors.append(np.random.rand(3))
    else:
        assert len(subgroup_colors) == nsubgroups, "subgroup_colors length must match number of subgroups (%d)" % nsubgroups

    def _decorate_box(_bp, _d):
        plt.setp(_bp['boxes'], lw=0, color='k')
        plt.setp(_bp['whiskers'], lw=3.0, color='k')

        # fill in each box with a color
        assert len(_bp['boxes']) == nsubgroups
        for _k,_box in enumerate(_bp['boxes']):
            _boxX = list()
            _boxY = list()
            for _j in range(5):
                _boxX.append(_box.get_xdata()[_j])
                _boxY.append(_box.get_ydata()[_j])
            _boxCoords = zip(_boxX, _boxY)
            _boxPolygon = plt.Polygon(_boxCoords, facecolor=subgroup_colors[_k])
            ax.add_patch(_boxPolygon)

        # draw a black line for the median
        for _k,_med in enumerate(_bp['medians']):
            _medianX = list()
            _medianY = list()
            for _j in range(2):
                _medianX.append(_med.get_xdata()[_j])
                _medianY.append(_med.get_ydata()[_j])
                plt.plot(_medianX, _medianY, 'k', linewidth=3.0)

            # draw a black asterisk for the mean
            plt.plot([np.mean(_med.get_xdata())], [np.mean(_d[_k])], color='w', marker='*',
                      markeredgecolor='k', markersize=12)

    cpos = 1
    label_pos = list()
    for k in group_names:
        d = data[k]
        nsubgroups = len(d)
        pos = np.arange(nsubgroups) + cpos
        label_pos.append(pos.mean())
        bp = plt.boxplot(d, positions=pos, widths=box_width)
        _decorate_box(bp, d)
        cpos += nsubgroups + box_spacing

    plt.xlim(0, cpos-1)
    plt.xticks(label_pos, group_names)

    if subgroup_names is not None:
        leg = custom_legend(subgroup_colors, subgroup_names)
        plt.legend(handles=leg, loc=legend_loc)


def boxplot_with_colors(data, group_names=None,ax=None, group_colors=None, box_width=0.6, box_spacing=1.0, box_alpha=1.):

    assert isinstance(data, dict)

    if group_names is None:
        group_names = data.keys()
    ngroups = len(data)

    if group_colors is None:
        group_colors = ['w']*ngroups

    if ax is None:
        ax = plt.gca()

    def _decorate_box(_bp, _d):

        plt.setp(_bp['boxes'], lw=0, color='k')
        plt.setp(_bp['whiskers'], lw=3.0, color='k')

        # fill in each box with a color
        assert len(_bp['boxes']) == ngroups
        for _k, _box in enumerate(_bp['boxes']):
            _boxX = list()
            _boxY = list()
            for _j in range(5):
                _boxX.append(_box.get_xdata()[_j])
                _boxY.append(_box.get_ydata()[_j])
            _boxCoords = zip(_boxX, _boxY)
            _boxPolygon = plt.Polygon(_boxCoords, facecolor=group_colors[_k], alpha=box_alpha)
            ax.add_patch(_boxPolygon)

        # draw a black line for the median
        for _k, _med in enumerate(_bp['medians']):
            _medianX = list()
            _medianY = list()
            for _j in range(2):
                _medianX.append(_med.get_xdata()[_j])
                _medianY.append(_med.get_ydata()[_j])
                plt.plot(_medianX, _medianY, 'k', linewidth=3.0)

            # draw a black asterisk for the mean
            plt.plot([np.mean(_med.get_xdata())], [np.mean(_d[_k])], color='w', marker='*',
                     markeredgecolor='k', markersize=12)

    pos = np.arange(ngroups) + 1
    d = [data[gname] for gname in group_names]
    bp = plt.boxplot(d, positions=pos, widths=box_width)
    _decorate_box(bp, d)

    plt.xlim(0.5, pos.max() + 0.5)
    plt.xticks(pos, group_names)


def compute_mean_from_scatter(x, y, bins=20, num_smooth_points=0, bin_by_quantile=False):
    assert len(x) == len(y)

    xcenter = np.zeros([bins])
    ymean = np.zeros([bins])
    yerr = np.zeros([bins])

    # bin the data, compute the mean and standard error of y for each bin
    if bin_by_quantile:
        p = 100. / (bins + 1)
        hist_edges = list([np.percentile(x, int((k + 1) * p)) for k in range(bins-1)])
        if len(np.unique(hist_edges)) != len(hist_edges):
            '[compute_mean_from_scatter] number of unique quantiles is less than the number of bins, defaulting to use hist'
            hist, hist_edges = np.histogram(x, bins=bins)
        else:
            hist_edges.insert(0, x.min())
            hist_edges.append(x.max())
            hist = None # unused
    else:
        hist, hist_edges = np.histogram(x, bins=bins)

    for k in range(bins):
        start = hist_edges[k]
        end = hist_edges[k + 1]
        i = (x >= start) & (x < end)

        xcenter[k] = ((end - start) / 2.) + start
        # print 'k=%d, start=%f, end=%f, xcenter=%f' % (k, start, end, xcenter[k])
        ymean[k] = y[i].mean()
        yerr[k] = y[i].std(ddof=1) / np.sqrt(i.sum())

    ymean_cs = None
    if num_smooth_points > 0:
        # interpolate the mean and sd with a cubic spline and resample
        x_rs = np.linspace(xcenter.min(), xcenter.max(), num_smooth_points)
        ymean_cs = interp1d(xcenter, ymean, kind='cubic')
        yerr_cs = interp1d(xcenter, yerr, kind='cubic')

        xcenter = x_rs
        ymean = ymean_cs(x_rs)
        yerr = yerr_cs(x_rs)

    return xcenter, ymean, yerr, ymean_cs


def plot_mean_from_scatter(x, y, bins=20, num_smooth_points=0,
                           color='k', ecolor='#D8D8D8', linewidth=4., elinewidth=3.0, alpha=0.5,
                           bin_by_quantile=False):
    """ For scatterplot data x,y, bin x, and plot the mean and standard error for each bin with respect to y. """

    xcenter,ymean,yerr,ymean_cs = compute_mean_from_scatter(x, y, bins, num_smooth_points, bin_by_quantile=bin_by_quantile)
    plt.errorbar(xcenter, ymean, yerr=yerr, c=color, linewidth=linewidth, elinewidth=elinewidth,
                 ecolor=ecolor, alpha=alpha, capthick=0.)


def plot_x_samps(x, y=0.):
    """ Plot ticks along the x axis at the bottom of a plot, one tick per sample. """

    plt.plot()


if __name__ == '__main__':

    # draw_husl_circle()

    data = { 'A':[np.random.randn(100), np.random.randn(100) + 5],
             'B':[np.random.randn(100)+1, np.random.randn(100) + 9],
             'C':[np.random.randn(100)-3, np.random.randn(100) -5]
           }

    grouped_boxplot(data, group_names=['A', 'B', 'C'], subgroup_names=['Apples', 'Oranges'], subgroup_colors=['#D02D2E', '#D67700'])
    plt.show()


def plot_binary_matrix(t, B, ax=None):
    """ Plot a binary matrix using patches, to make sure nothing disappears due to interpolation...

    :param t: The timestamp for each column of B, of length num_time_points
    :param B: The binary matrix of shape (ncols, num_time_points)
    :param ax: The axis to plot on (default=current axis)
    :return:
    """

    if ax is not None:
        plt.sca(ax)

    # first plot grid



    # now plot rectangles for each event








import numpy as np

import matplotlib.pyplot as plt
from soundsig.plots import multi_plot

"""
Implementation of S. Zayd Enam's STRF modeling stuff:

S. Zayd Enam, Michael R. DeWeese, "Spectro-Temporal Models of Inferior Colliculus Neuron Receptive Fields"
http://users.soe.ucsc.edu/~afletcher/hdnips2013/papers/strfmodels_plos.pdf

"""


def onset_strf(t, f, t_c=0.150, t_freq=10.0, t_phase=0.0, t_sigma=0.250, f_c=3000.0, f_sigma=500.0):

    T,F = np.meshgrid(t, f)
    f_part = np.exp(-(F - f_c)**2 / (2*f_sigma**2))
    t_part = np.sin(2*np.pi*t_freq*(T - t_c) + t_phase)
    exp_part = np.exp(  (-(T - t_c)**2 / (2*t_sigma**2)) )

    strf = t_part*f_part*exp_part
    strf /= np.abs(strf).max()
    return strf


def checkerboard_strf(t, f, t_freq=10.0, t_phase=0.0,
                      f_freq=1e-6, f_phase=0.0, t_c=0.150, f_c=3000.0,
                      t_sigma=0.050, f_sigma=500.0, harmonic=False):

    T,F = np.meshgrid(t, f)
    t_part = np.cos(2*np.pi*t_freq*T + t_phase)
    f_part = np.cos(2*np.pi*f_freq*F + f_phase)
    exp_part = np.exp(  (-(T-t_c)**2 / (2*t_sigma**2)) - ((F - f_c)**2 / (2*f_sigma**2)) )

    if harmonic:
        f_part = np.abs(f_part)

    strf = t_part*f_part*exp_part
    strf /= np.abs(strf).max()
    return strf


def sweep_strf(t, f, theta=0.0, aspect_ratio=1.0, phase=0.0, wavelength=0.5, spread=1.0, f_c=5000.0, t_c=0.0):

    T,F = np.meshgrid(t-t_c, f-f_c)
    T /= np.abs(T).max()
    F /= np.abs(F).max()

    Tp = T*np.cos(theta) + F*np.sin(theta)
    Fp = -T*np.sin(theta) + F*np.cos(theta)

    exp_part = np.exp( -(Tp**2 + (aspect_ratio**2 * Fp**2)) / (2*spread**2) )
    cos_part = np.cos( (2*np.pi*Tp / wavelength) + phase)

    return exp_part*cos_part


def plot_strf(pdata, ax):
    strf = pdata['strf']
    absmax = np.abs(strf).max()
    plt.imshow(strf, interpolation='nearest', aspect='auto', origin='lower',
               extent=plot_extent, vmin=-absmax, vmax=absmax, cmap=plt.cm.seismic)
    plt.title(pdata['title'])
    plt.xticks([])
    plt.yticks([])


if __name__ == '__main__':

    nt = 100
    t = np.linspace(0.0, 0.250)
    nf = 100
    f = np.linspace(300.0, 8000.0, nf)
    plot_extent = [t.min(), t.max(), f.min(), f.max()]


    #build onset STRFs of varying center frequency and temporal bandwidths
    onset_f_sigma = 500
    onset_f_c = np.linspace(300.0, 8000.0, 10)
    onset_t_sigmas = np.array([0.005, 0.010, 0.025, 0.050])
    onset_t_freqs = np.array([20.0, 15.0, 10.0, 5.0])

    onset_plist = list()
    for f_c in onset_f_c:
        for t_sigma,t_freq in zip(onset_t_sigmas, onset_t_freqs):

            t_c = 0.5*(1.0 / t_freq) - 0.010
            strf = onset_strf(t, f, t_freq=t_freq, t_phase=np.pi, f_c=f_c, f_sigma=1000.0, t_sigma=t_sigma, t_c=t_c)
            title = '$f_c$=%dHz, $\sigma_t$=%dms, $f_t$=%dHz' % (f_c, t_sigma*1e3, t_freq)
            onset_plist.append({'strf':strf, 'title':title})

    multi_plot(onset_plist, plot_strf, nrows=len(onset_f_c), ncols=len(onset_t_sigmas))


    #build harmonic stack STRFs
    stack_t_sigma = 0.005
    stack_f_sigma = 1500
    stack_f_c = np.linspace(300.0, 8000.0, 10)
    stack_f_freq = np.linspace(1e-4, 7e-4, 5)

    stack_t_freqs = np.array([20.0, 15.0, 10.0, 5.0])

    stack_plist = list()
    for f_c in stack_f_c:
        for f_freq in stack_f_freq:
            strf = checkerboard_strf(t, f,
                                     t_freq=10.0, t_phase=0.0,
                                     f_freq=f_freq, f_phase=0.0,
                                     t_c=0.015, f_c=f_c,
                                     t_sigma=stack_t_sigma, f_sigma=stack_f_sigma, harmonic=False)

            title = '$f_c$=%dHz, f_freq=%0.6f' % (f_c, f_freq)
            stack_plist.append({'strf':strf, 'title':title})

    multi_plot(stack_plist, plot_strf, nrows=len(stack_f_c), ncols=len(stack_f_freq))

    #build frequency sweep STRFs
    sweep_wavelengths = np.array([0.25, 0.5, 0.75])
    sweep_spreads = np.array([0.100, 0.150, 0.200, 0.250])
    sweep_thetas = np.array([-np.pi/8, -np.pi/6, -np.pi/4, np.pi/4, np.pi/6, np.pi/8])

    sweep_plist = list()
    for wavelength,spread in zip(sweep_wavelengths, sweep_spreads):
        for theta in sweep_thetas:

            t_c = 0.1*wavelength
            strf = sweep_strf(t, f, theta=theta, wavelength=wavelength, spread=spread, t_c=t_c)
            title = '$\lambda$=%0.3f, $\\theta$=%d$\degree$' % (wavelength, theta*(180.0 / np.pi))
            sweep_plist.append({'strf':strf, 'title':title})

    multi_plot(sweep_plist, plot_strf, nrows=len(sweep_wavelengths), ncols=len(sweep_thetas))

    plt.show()
